// This program publishes randomly-generated velocity
// messages for turtlesim.
#include <ros/ros.h>
#include <turtlesim/Pose.h>
#include <geometry_msgs/Twist.h>  // For geometry_msgs::Twist
#include <stdlib.h> // For rand() and RAND_MAX

int safezone;
double max=8;
double min=3;

// A callback function.  Executed each time a new pose
// message arrives.
void poseMessageReceived(const turtlesim::Pose& msg) {

  if(msg.x>=max || msg.y>=max || msg.x<=min || msg.y<=min ) safezone = 0;
  else safezone=1;
  //ROS_INFO_STREAM(std::setprecision(2) << std::fixed
  //  << "position=(" <<  msg.x << "," << msg.y << ")"
  //  << " direction=" << msg.theta);
}

int main(int argc, char **argv) {
  // Initialize the ROS system and become a node.
  ros::init(argc, argv, "publish_safe_velocity");
  ros::NodeHandle nh;
  safezone=1;

  // Create a publisher object.
  ros::Publisher pub = nh.advertise<geometry_msgs::Twist>(
    "turtle1/cmd_vel", 1000);

  // Create a subscriber object.
  ros::Subscriber sub = nh.subscribe("turtle1/pose", 1000,
    &poseMessageReceived);

  // Seed the random number generator.
  srand(time(0));

  // Loop at 2Hz until the node is shut down.
  ros::Rate rate(2);
  while(ros::ok()) {
    // Create and fill in the message.  The other four
    // fields, which are ignored by turtlesim, default to 0.
    geometry_msgs::Twist msg;
    if(safezone==1) msg.linear.x = 1.0;
    else msg.linear.x = double(rand())/double(RAND_MAX);
    msg.angular.z = 2*double(rand())/double(RAND_MAX) - 1;

    // Publish the message.
    pub.publish(msg);

    // Send a message to rosout with the details.
    ROS_INFO_STREAM("Sending random velocity command:"
      << " linear=" << msg.linear.x
      << " angular=" << msg.angular.z);

    ros::spinOnce();
    // Wait until it's time for another iteration.
    rate.sleep();
  }
}
