// This program publishes randomly-generated velocity
// messages for turtlesim.
#include <ros/ros.h>
#include <turtlesim/Pose.h>
#include <geometry_msgs/Twist.h>  // For geometry_msgs::Twist
#include <stdlib.h> // For rand() and RAND_MAX

int safe;
int turn;

// A callback function.  Executed each time a new pose
// message arrives.
void poseMessageReceived(const turtlesim::Pose& msg) {
  if((msg.x < 9 && msg.x>2)&& (msg.y < 9 && msg.y>2)) {
      safe=1;
      ROS_INFO_STREAM("...SAFE...");
  }
  else
  {
    safe=0;


    if((msg.x>10.5 && (msg.theta>-M_PI/2 && msg.theta<M_PI/2)) ||
       (msg.x<0.5 && (msg.theta>M_PI/2 || msg.theta<-M_PI/2)) ||
       (msg.y>10.5 && (msg.theta>0 && msg.theta<M_PI)) ||
       (msg.y<0.5 && (msg.theta<0 && msg.theta>-M_PI))) turn=1;
    else turn=0;

    if(turn==0)ROS_INFO_STREAM("...NOT SAFE...");
    else ROS_INFO_STREAM("...DANGER!! - TURNING ROUND...");


    /*ROS_INFO_STREAM("NOT safe:"
      << " x=" << msg.x
      << " y=" << msg.y
      << " theta=" << msg.theta
      << " TURN=" << turn);*/

  }
}

int main(int argc, char **argv) {
  // Initialize the ROS system and become a node.
  ros::init(argc, argv, "publish_safe_velocity_b");
  ros::NodeHandle nh;

  safe=0;
  // Create a publisher object.
  ros::Publisher pub = nh.advertise<geometry_msgs::Twist>(
    "turtle1/cmd_vel", 1000);

  // Create a subscriber object.
  ros::Subscriber sub = nh.subscribe("turtle1/pose", 1000,
    &poseMessageReceived);

  // Seed the random number generator.
  srand(time(0));

  // Loop at 2Hz until the node is shut down.
  ros::Rate rate(2);
  while(ros::ok()) {
    // Create and fill in the message.  The other four
    // fields, which are ignored by turtlesim, default to 0.
    geometry_msgs::Twist msg;
    if(safe==0)
    {
        if(turn)
        {
            msg.linear.x = 0;
            msg.angular.z = - M_PI;
        }
        else{
            msg.linear.x = double(rand())/double(RAND_MAX);
            msg.angular.z = 2*double(rand())/double(RAND_MAX) - 1;
        }
    }
    else
    {
      msg.linear.x = 1;
      msg.angular.z = 2*double(rand())/double(RAND_MAX) - 1;
    }
    // Publish the message.
    pub.publish(msg);

    // Send a message to rosout with the details.
    ROS_INFO_STREAM("Sending random velocity command:"
      << " linear=" << msg.linear.x
      << " angular=" << msg.angular.z);


    // Wait until it's time for another iteration.
    rate.sleep();
    // Let ROS take over.
    ros::spinOnce();
  }
}
